@@ -2,8 +2,10 @@
 #define ID_RANDOM_UTIL_HPP_
 #include "BulletInverseDynamics/IDConfig.hpp"
 namespace btInverseDynamics {
-/// seed random number generator
+/// seed random number generator using time()
 void randomInit();
+/// seed random number generator with identical value to get repeatable results
+void randomInit(unsigned seed);
 /// Generate (not quite) uniformly distributed random integers in [low, high]
 /// Note: this is a low-quality implementation using only rand(), as
 /// C++11 <random> is not supported in bullet.
