@@ -1,8 +1,9 @@
 
 include "HACD"
+include "VHACD"
 include "ConvexDecomposition"
 include "InverseDynamics"
 include "Serialize/BulletFileLoader"
 include "Serialize/BulletWorldImporter"
 include "Serialize/BulletXmlWorldImporter"
-
+include "obj2sdf"
