@@ -68,6 +68,10 @@ typedef btScalar idScalar;
 #ifdef BT_USE_DOUBLE_PRECISION
 #define BT_ID_USE_DOUBLE_PRECISION
 #endif
+
+#ifndef BT_USE_INVERSE_DYNAMICS_WITH_BULLET2
+
+
 // use bullet types for arrays and array indices
 #include "Bullet3Common/b3AlignedObjectArray.h"
 // this is to make it work with C++2003, otherwise we could do this:
@@ -78,7 +82,20 @@ struct idArray {
 	typedef b3AlignedObjectArray<T> type;
 };
 typedef int idArrayIdx;
-#define ID_DECLARE_ALIGNED_ALLOCATOR B3_DECLARE_ALIGNED_ALLOCATOR
+#define ID_DECLARE_ALIGNED_ALLOCATOR() B3_DECLARE_ALIGNED_ALLOCATOR()
+
+#else // BT_USE_INVERSE_DYNAMICS_WITH_BULLET2
+
+#include "LinearMath/btAlignedObjectArray.h"
+template <typename T>
+struct idArray {
+	typedef btAlignedObjectArray<T> type;
+};
+typedef int idArrayIdx;
+#define ID_DECLARE_ALIGNED_ALLOCATOR() BT_DECLARE_ALIGNED_ALLOCATOR()
+
+#endif // BT_USE_INVERSE_DYNAMICS_WITH_BULLET2
+
 
 // use bullet's allocator functions
 #define idMalloc btAllocFunc
