@@ -15,13 +15,14 @@
 		"../../examples/ThirdPartyLibs",
 		"../../examples",	
 		".",
+		"../../src"
 	}
 
 	initOpenGL()
 	initGlew()
 			
 	links {
-		"gwen",
+		"gwen","Bullet3Common"
 	}
 	
 	
@@ -43,6 +44,9 @@
 		"**.cpp",
 		"**.h",
 	}
+	files {
+		"../../examples/OpenGLWindow/GLFWOpenGLWindow.cpp",
+		}
 	if os.is("Windows") then
 	files {
 		"../../examples/OpenGLWindow/Win32OpenGLWindow.cpp",
@@ -61,9 +65,10 @@
 	end
 	if os.is("MacOSX") then
 		links{"Cocoa.framework"}
-print("hello!")
 		files{
-		"../../examples/OpenGLWindow/MacOpenGLWindow.mm",
+		"../../examples/OpenGLWindow/MacOpenGLWindow.cpp",
 		"../../examples/OpenGLWindow/MacOpenGLWindow.h",
+		"../../examples/OpenGLWindow/MacOpenGLWindowObjC.m",
+		"../../examples/OpenGLWindow/MacOpenGLWindowObjC.h",
 		}
 	end
